# Copyright (c) 2023 ZKM | Hertz-Lab
# Dan Wilcox <dan.wilcox@zkm.de>
#
# BSD Simplified License.
# For information on usage and redistribution, and for a DISCLAIMER OF ALL
# WARRANTIES, see the file, "LICENSE.txt," in this distribution.
#
# This code has been developed at ZKM | Hertz-Lab as part of „The Intelligent
# Museum“ generously funded by the German Federal Cultural Foundation.

# FIXME: hack to find shared module
import os, sys
sys.path.append(os.path.join(os.path.dirname(__file__), "../"))

import things

##### helpers

# if set, add order by to query string and param args
# assumes only valid values are "asc" or "desc"
def _add_order(q, args, order):
    if order:
        q += " ORDER BY id " + order.upper()
    return (q, args) 

# if > 0, add limit to query string and param args
def _add_limit(q, args, limit):
    if limit > 0:
        q += " LIMIT ?"
        if not args: args = []
        args.append(limit)
    return (q, args)

##### queries

# count messages
def count_messages(db):
    rows = db.query("SELECT COUNT(*) FROM " + things.schema.TABLE + ";")
    if not rows: rows = [(0)]
    return rows[0][0]

# select messages
def select_messages(db, limit=0, order=None):
    q = "SELECT time, topic, message FROM " + things.schema.TABLE
    args = None
    q, args = _add_order(q, args, order)
    q, args = _add_limit(q, args, limit)
    q += ";"
    print(q, args)
    rows = db.query(q, args)
    if not rows: rows = []
    return rows

# select current message for each topic
def select_current_messages(db):
    rows = db.query("SELECT MAX(time) AS time, topic, message AS time FROM " + things.schema.TABLE + " GROUP BY topic")
    if not rows: rows = []
    return rows

# select unique topics and and message count
def select_topics(db):
    rows = db.query("SELECT topic, COUNT(topic) as 'count' FROM " + things.schema.TABLE + " GROUP BY topic;")
    if not rows: rows = []
    return rows

# select messages by topic match
def match_topics(db, topics, limit=0, order=None):
    if len(topics) < 1: return []
    q = "SELECT time, topic, message FROM " + things.schema.TABLE + " WHERE "
    q += "topic LIKE ?" # first topic
    for i in range(1, len(topics)): # second, third, ...
        q += " OR topic LIKE ?"
    args = topics
    q, args = _add_order(q, args, order)
    q, args = _add_limit(q, args, limit)
    q += ";"
    rows = db.query(q, args)
    if not rows: rows = []
    return rows
