// Copyright (c) 2023 ZKM | Hertz-Lab
// Dan Wilcox <dan.wilcox@zkm.de>
//
// BSD Simplified License.
// For information on usage and redistribution, and for a DISCLAIMER OF ALL
// WARRANTIES, see the file, "LICENSE.txt," in this distribution.
//
// This code has been developed at ZKM | Hertz-Lab as part of „The Intelligent
// Museum“ generously funded by the German Federal Cultural Foundation.

// Referencess:
// * https://d3-graph-gallery.com/graph/line_basic.html
// * http://using-d3js.com/05_04_curves.html

import Widget from "./Widget.js"

// lidar widget: displays current tracked people
export class Graph extends Widget {

  // optional args:
  // * maxlen: max length of graph
  constructor(args) {
    super(args)
    this.data = []
    this.maxlen = (args.maxlen === undefined ? 10 : args.maxlen)
    this.scale.x.domain([0, this.maxlen-1]) // data min max

    let self = this
    this.linegen = d3.line()
      .curve(d3.curveLinear)
      .x(function(d, i){return self.scale.x(i)})
      .y(function(d){return self.scale.y(d)})
  }

  // setup div with format:
  // <div id="lidar" class="widget widget-graph">
  //     <svg id="lidar-svg" class="widget-svg" viewBox="0 0 W H"
  // preserveAspectRatio="xMidYMid meet"></svg>
  //     <div class="widget-title">LIDAR</div>
  //     <div class="widget-meta"><span class="widget-value">0</span> %</div>
  //     <div class="widget-controls">
  //         <button type="button">clear</button>
  //     </div>
  // </div>
  setup(w, h) {
    let self = this
    super.setup(w, h)

    this.path = this.svg.append("path")
      .style("stroke", "green")
      .style("stroke-width", 3)
      .style("fill", "none")
    // TODO axes + margin?
    //this.svg.call(d3.axisLeft(this.scale.y)) // set axis from domain

    this.div.classed("widget-graph", true)

    this.div.selectAll(".widget-controls")
      .append("button")
        .attr("type", "button")
        .on("click", function() {self.clear()})
        .text("clear")
  }

  // receive JSON message as a person
  receive(obj) {
    this.data.push(obj)
    while(this.data.length > this.maxlen) {
      this.data.shift()
    }
    this.update()
  }

  // update svg from current data
  update() {
    let self = this
    this.path.attr("d", self.linegen(self.data))
    this.setValue(this.data[this.data.length-1])
  }

  set(data) {
    this.data = data
    this.update()
  }

  clear() {
    this.data = []
    this.update()
  }

}

export default Graph
