// Copyright (c) 2023 ZKM | Hertz-Lab
// Dan Wilcox <dan.wilcox@zkm.de>
//
// BSD Simplified License.
// For information on usage and redistribution, and for a DISCLAIMER OF ALL
// WARRANTIES, see the file, "LICENSE.txt," in this distribution.
//
// This code has been developed at ZKM | Hertz-Lab as part of „The Intelligent
// Museum“ generously funded by the German Federal Cultural Foundation.

// use localhost when only connecting internally
let host = "localhost"
let port = 9001
let topics = ["lidar/person", "lidar/count"]
let client_id = "" // note: set to "" for server-generated id
let client = null // mqtt websocket

export function setup() {
  client = new Paho.Client(host, port, "lidar-web")
  client.onConnectionLost = (response) => {
    if(response.errorCode !== 0) {
      console.log("mqtt: disconnected: " + response.errorMessage)
    }
    status.id.textContent = "disconnected"
  }
  client.onMessageArrived = (message) => {
    //console.log("mqtt recv: ", message.destinationName, " ", message.payloadString)
    let data = JSON.parse(message.payloadString)
    if(data) {
      messageReceived(message.destinationName, data)
    }
  }
}

export function connect() {
  client.connect({
    onSuccess: () => {
      console.log("mqtt: connected")
      for(let t of topics) {
        client.subscribe(t)
      }
      status.id.textContent = "connected"
    },
    onFailure: (response) => {
      console.error("mqtt: connect failed: " + response.errorMessage)
      status.id.textContent = "connect failed"
    },
    cleanSession: true,
    reconnect: true
  })
}

export function disconnect() {
    client.disconnect()
}